@@ -21,17 +21,22 @@
    Author: Jaka Mocnik <jaka@gnu.org>
 */
 
-#ifndef __GHEX_CHARTABLE_H__
-#define __GHEX_CHARTABLE_H__
+#ifndef GHEX_CHARTABLE_H
+#define GHEX_CHARTABLE_H
+
+#include <stdlib.h>
 
 #include <gtk/gtk.h>
+#include <glib/gi18n.h>
+
+#include "gtkhex.h"
+#include "common-ui.h"
 
 G_BEGIN_DECLS
 
-GtkWidget *create_char_table(void);
-
-extern GtkWidget *char_table;
+GtkWidget *create_char_table (GtkWindow *parent_win, /* can-NULL */
+		HexWidget *gh);
 
 G_END_DECLS
 
-#endif /* !__GHEX_CHARTABLE_H__ */
+#endif /* GHEX_CHARTABLE_H */
