@@ -2,7 +2,7 @@
  * -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* ghex-application-window.c - GHex main application window
 
-   Copyright © 2021 Logan Rathbone <poprocks@gmail.com>
+   Copyright © 2021-2023 Logan Rathbone <poprocks@gmail.com>
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -68,6 +68,8 @@ struct _GHexApplicationWindow
 	GtkWidget *find_dialog;
 	GtkWidget *replace_dialog;
 	GtkWidget *jump_dialog;
+	GtkWidget *mark_dialog;
+
 	GtkWidget *chartable;
 	GtkWidget *converter;
 	GtkWidget *prefs_dialog;
@@ -97,6 +99,7 @@ typedef enum
 	PROP_FIND_OPEN,
 	PROP_REPLACE_OPEN,
 	PROP_JUMP_OPEN,
+	PROP_MARK_OPEN,
 	PROP_CAN_SAVE,
 	PROP_INSERT_MODE,
 	N_PROPERTIES
@@ -126,6 +129,7 @@ static const char *main_actions[] = {
 	"ghex.converter",
 	"ghex.copy-special",
 	"ghex.paste-special",
+	"ghex.mark-dialog",
 	NULL				/* last action */
 };
 
@@ -147,8 +151,11 @@ static void ghex_application_window_set_show_replace (GHexApplicationWindow *sel
 		gboolean show);
 static void ghex_application_window_set_show_jump (GHexApplicationWindow *self,
 		gboolean show);
+static void ghex_application_window_set_show_mark (GHexApplicationWindow *self,
+		gboolean show);
 static void ghex_application_window_set_can_save (GHexApplicationWindow *self,
 		gboolean can_save);
+static gboolean get_pane_visible (GHexApplicationWindow *self, PaneDialog *pane);
 
 static void enable_main_actions (GHexApplicationWindow *self, gboolean enable);
 static void update_status_message (GHexApplicationWindow *self);
@@ -260,6 +267,8 @@ refresh_dialogs (GHexApplicationWindow *self)
 		pane_dialog_set_hex (PANE_DIALOG(self->find_dialog), ACTIVE_GH);
 		pane_dialog_set_hex (PANE_DIALOG(self->replace_dialog), ACTIVE_GH);
 		pane_dialog_set_hex (PANE_DIALOG(self->jump_dialog), ACTIVE_GH);
+		pane_dialog_set_hex (PANE_DIALOG(self->mark_dialog), ACTIVE_GH);
+		mark_dialog_refresh (MARK_DIALOG(self->mark_dialog));
 	}
 }
 
@@ -286,7 +295,7 @@ file_save_write_cb (HexDocument *doc,
 		if (local_error)
 			g_string_append_printf (full_errmsg, "\n\n%s", local_error->message);
 
-		display_error_dialog (GTK_WINDOW(self), full_errmsg->str);
+		display_dialog (GTK_WINDOW(self), full_errmsg->str);
 
 		g_string_free (full_errmsg, TRUE);
 	}
@@ -307,42 +316,37 @@ file_save (GHexApplicationWindow *self)
 }
 
 static void
-close_all_tabs_response_cb (GtkDialog *dialog,
-		int        response_id,
-		gpointer   user_data)
+close_all_tabs_response_cb (AdwMessageDialog *dialog,
+		const char *response,
+		GHexApplicationWindow *self)
 {
-	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
-	
 	/* Regardless of what the user chose, get rid of the dialog. */
-	gtk_window_destroy (GTK_WINDOW(dialog));
+	gtk_window_close (GTK_WINDOW(dialog));
 
-	if (response_id == GTK_RESPONSE_ACCEPT)
-	{
+	if (g_strcmp0 (response, "accept") == 0)
 		gtk_window_destroy (GTK_WINDOW(self));
-	}
 }
 
 static void
 close_all_tabs_confirmation_dialog (GHexApplicationWindow *self)
 {
-	GtkWidget *dialog;
+	GtkWidget *dialog = adw_message_dialog_new (GTK_WINDOW(self), NULL, NULL);
 
-	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(self),
-			GTK_DIALOG_MODAL,
-			GTK_MESSAGE_QUESTION,
-			GTK_BUTTONS_NONE,
-			(_("<b>You have one or more files open with unsaved changes.</b>\n\n"
-			   "Are you sure you want to close the window?\n\n")));
-
-	gtk_dialog_add_buttons (GTK_DIALOG(dialog),
-			_("_Close Anyway"),		GTK_RESPONSE_ACCEPT,
-			_("_Go Back"),			GTK_RESPONSE_REJECT,
+	adw_message_dialog_format_body_markup (ADW_MESSAGE_DIALOG(dialog),
+			_("<b>You have one or more files open with unsaved changes.</b>\n\n"
+			   "Are you sure you want to close the window?\n\n"));
+	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG(dialog),
+			"accept", _("_Close Anyway"),
+			"reject", _("_Go Back"),
 			NULL);
+	/* workaround since AdwMessageDialog sets destroy-with-parent to true, which
+	 * can cause runtime errors upon exit. See: https://discourse.gnome.org/t/glib-gobject-critical-and-glib-gobject-warning-messages/9909
+	 */
+	gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog), FALSE);
 
-	g_signal_connect (dialog, "response",
-			G_CALLBACK(close_all_tabs_response_cb), self);
+	g_signal_connect (dialog, "response", G_CALLBACK(close_all_tabs_response_cb), self);
 
-	gtk_widget_show (dialog);
+	gtk_window_present (GTK_WINDOW(dialog));
 }
 
 static void
@@ -393,6 +397,7 @@ close_page_finish_helper (GHexApplicationWindow *self, AdwTabView *tab_view, Adw
 		ghex_application_window_set_show_find (self, FALSE);
 		ghex_application_window_set_show_replace (self, FALSE);
 		ghex_application_window_set_show_jump (self, FALSE);
+		ghex_application_window_set_show_mark (self, FALSE);
 		ghex_application_window_set_show_chartable (self, FALSE);
 		ghex_application_window_set_show_converter (self, FALSE);
 
@@ -403,20 +408,19 @@ close_page_finish_helper (GHexApplicationWindow *self, AdwTabView *tab_view, Adw
 }
 
 static void
-close_doc_response_cb (GtkDialog *dialog,
-		int        response_id,
-		gpointer   user_data)
+close_doc_response_cb (AdwMessageDialog *dialog,
+		const char *response,
+		GHexApplicationWindow *self)
 {
-	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
 	AdwTabView *tab_view = ADW_TAB_VIEW(self->hex_tab_view);
 	AdwTabPage *page = g_object_get_data (G_OBJECT(self), "target-page");
 	
-	if (response_id == GTK_RESPONSE_ACCEPT)
+	if (g_strcmp0 (response, "accept") == 0)
 	{
 		file_save (self);
 		close_page_finish_helper (self, tab_view, page, TRUE);
 	}
-	else if (response_id == GTK_RESPONSE_REJECT)
+	else if (g_strcmp0 (response, "reject") == 0)
 	{
 		close_page_finish_helper (self, tab_view, page, TRUE);
 	}
@@ -459,28 +463,23 @@ close_doc_confirmation_dialog (GHexApplicationWindow *self, AdwTabPage *page)
 			   "Would you like to save your changes?"));
 	}
 
-	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(self),
-			GTK_DIALOG_MODAL,
-			GTK_MESSAGE_QUESTION,
-			GTK_BUTTONS_NONE,
-			NULL);
-	gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog),
-			message);
+	dialog = adw_message_dialog_new (GTK_WINDOW(self), NULL, NULL);
+	adw_message_dialog_set_body_use_markup (ADW_MESSAGE_DIALOG(dialog), TRUE);
+	adw_message_dialog_set_body (ADW_MESSAGE_DIALOG(dialog), message);
+
 	g_free (message);
 
-	gtk_dialog_add_buttons (GTK_DIALOG(dialog),
-			_("_Save Changes"),		GTK_RESPONSE_ACCEPT,
-			_("_Discard Changes"),	GTK_RESPONSE_REJECT,
-			_("_Go Back"),			GTK_RESPONSE_CANCEL,
+	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG(dialog),
+			"accept", _("_Save Changes"),
+			"reject", _("_Discard Changes"),
+			"cancel", _("_Go Back"),
 			NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
+	adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG(dialog), "cancel");
+	g_signal_connect (dialog, "response", G_CALLBACK(close_doc_response_cb), self);
 
 	g_object_set_data (G_OBJECT(self), "target-page", page);
 
-	g_signal_connect (dialog, "response",
-			G_CALLBACK(close_doc_response_cb), self);
-
-	gtk_widget_show (dialog);
+	gtk_window_present (GTK_WINDOW(dialog));
 }
 
 static void
@@ -574,15 +573,15 @@ assess_can_save (HexDocument *doc)
 static void
 show_hex_tab_view (GHexApplicationWindow *self)
 {
-	gtk_widget_hide (self->no_doc_label);
-	gtk_widget_show (self->child_box);
+	gtk_widget_set_visible (self->no_doc_label, FALSE);
+	gtk_widget_set_visible (self->child_box, TRUE);
 }
 
 static void
 show_no_file_loaded_label (GHexApplicationWindow *self)
 {
-	gtk_widget_hide (self->child_box);
-	gtk_widget_show (self->no_doc_label);
+	gtk_widget_set_visible (self->child_box, FALSE);
+	gtk_widget_set_visible (self->no_doc_label, TRUE);
 }
 
 static void
@@ -760,24 +759,15 @@ pane_close_cb (PaneDialog *pane, gpointer user_data)
 	g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_FIND_OPEN]);
 	g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_REPLACE_OPEN]);
 	g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_JUMP_OPEN]);
+	g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_MARK_OPEN]);
 }
 
-static void
-chartable_close_cb (GtkWindow *window,
-		gpointer user_data)
+static gboolean
+chartable_close_cb (GHexApplicationWindow *self)
 {
-	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
-
 	ghex_application_window_set_show_chartable (self, FALSE);
-}
 
-static void
-converter_close_cb (GtkWindow *window,
-		gpointer user_data)
-{
-	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
-
-	ghex_application_window_set_show_converter (self, FALSE);
+	return GDK_EVENT_STOP;
 }
 
 static void
@@ -787,10 +777,18 @@ setup_chartable (GHexApplicationWindow *self)
 
 	self->chartable = create_char_table (GTK_WINDOW(self), ACTIVE_GH);
 
-    g_signal_connect(self->chartable, "close-request",
+    g_signal_connect_swapped (self->chartable, "close-request",
                      G_CALLBACK(chartable_close_cb), self);
 }
 
+static gboolean
+converter_close_cb (GHexApplicationWindow *self)
+{
+	ghex_application_window_set_show_converter (self, FALSE);
+
+	return GDK_EVENT_STOP;
+}
+
 static void
 setup_converter (GHexApplicationWindow *self)
 {
@@ -798,8 +796,8 @@ setup_converter (GHexApplicationWindow *self)
 
 	self->converter = create_converter (GTK_WINDOW(self), ACTIVE_GH);
 
-	g_signal_connect(self->converter, "close-request",
-                     G_CALLBACK(converter_close_cb), self);
+    g_signal_connect_swapped (self->converter, "close-request",
+			G_CALLBACK(converter_close_cb), self);
 }
 
 static void
@@ -904,7 +902,13 @@ dnd_drop_cb (GtkDropTarget *target, const GValue *value, double x, double y,
 {
 	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(data);
 
-	ghex_application_window_open_file (self, g_value_get_object (value));
+	GSList *file_list = g_value_get_boxed (value);
+	GSList *l;
+	for (l = file_list; l; l = l->next)
+	{
+		GFile *file = l->data;
+		ghex_application_window_open_file (self, file);
+	}
 
 	return TRUE;
 }
@@ -921,13 +925,13 @@ ghex_application_window_set_show_ ##WIDGET (GHexApplicationWindow *self,	\
 		if (! GTK_IS_WIDGET(self->WIDGET)) {								\
 			SETUP_FUNC;														\
 		}																	\
-		gtk_widget_show (self->WIDGET);										\
+		gtk_widget_set_visible (self->WIDGET, TRUE);						\
 	}																		\
 	else																	\
 	{																		\
 		if (GTK_IS_WIDGET (self->WIDGET) &&									\
 				gtk_widget_is_visible (self->WIDGET)) {						\
-			gtk_widget_hide (self->WIDGET);									\
+			gtk_widget_set_visible (self->WIDGET, FALSE);					\
 			gtk_widget_grab_focus (GTK_WIDGET(ACTIVE_GH));					\
 		}																	\
 	}																		\
@@ -940,15 +944,16 @@ DIALOG_SET_SHOW_TEMPLATE(converter, setup_converter(self), PROP_CONVERTER_OPEN)
 /* Note that in this macro, it's up to the "closed" cb functions to notify
  * by pspec
  */
-#define PANE_SET_SHOW_TEMPLATE(WIDGET, OTHER1, OTHER2)								\
+#define PANE_SET_SHOW_TEMPLATE(WIDGET, OTHER1, OTHER2, OTHER3)						\
 static void																			\
 ghex_application_window_set_show_ ##WIDGET (GHexApplicationWindow *self,			\
 		gboolean show)																\
 {																					\
 	if (show) {																		\
-		gtk_widget_hide (self->OTHER1 ## _dialog);									\
-		gtk_widget_hide (self->OTHER2 ## _dialog);									\
-		gtk_widget_show (self->WIDGET ## _dialog);									\
+		gtk_widget_set_visible (self->OTHER1 ## _dialog, FALSE);					\
+		gtk_widget_set_visible (self->OTHER2 ## _dialog, FALSE);					\
+		gtk_widget_set_visible (self->OTHER3 ## _dialog, FALSE);					\
+		gtk_widget_set_visible (self->WIDGET ## _dialog, TRUE);						\
 		gtk_widget_grab_focus (self->WIDGET ## _dialog);							\
 		if (! gtk_revealer_get_reveal_child (GTK_REVEALER(self->findreplace_revealer)))	\
 		{																			\
@@ -959,14 +964,16 @@ ghex_application_window_set_show_ ##WIDGET (GHexApplicationWindow *self,			\
 		g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_FIND_OPEN]);		\
 		g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_REPLACE_OPEN]);	\
 		g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_JUMP_OPEN]);		\
+		g_object_notify_by_pspec (G_OBJECT(self), properties[PROP_MARK_OPEN]);		\
 	} else {																		\
 		g_signal_emit_by_name (self->WIDGET ## _dialog, "closed");					\
 	}																				\
 }												/* PANE_SET_SHOW_TEMPLATE */
 
-PANE_SET_SHOW_TEMPLATE(find,		replace, jump)
-PANE_SET_SHOW_TEMPLATE(replace,		find, jump)
-PANE_SET_SHOW_TEMPLATE(jump,		find, replace)
+PANE_SET_SHOW_TEMPLATE(find,		replace, jump, mark)
+PANE_SET_SHOW_TEMPLATE(replace,		find, jump, mark)
+PANE_SET_SHOW_TEMPLATE(jump,		find, replace, mark)
+PANE_SET_SHOW_TEMPLATE(mark,		find, replace, jump)
 
 /* Property setters without templates: */
 
@@ -1030,7 +1037,7 @@ save_as_write_to_file_cb (HexDocument *doc,
 		if (local_error)
 			g_string_append_printf (full_errmsg, "\n\n%s", local_error->message);
 
-		display_error_dialog (GTK_WINDOW(self), full_errmsg->str);
+		display_dialog (GTK_WINDOW(self), full_errmsg->str);
 
 		g_string_free (full_errmsg, TRUE);
 		return;
@@ -1043,12 +1050,13 @@ save_as_write_to_file_cb (HexDocument *doc,
 	}
 	else
 	{
-		display_error_dialog (GTK_WINDOW(self),
+		display_dialog (GTK_WINDOW(self),
 				_("An unknown error has occurred in attempting to reload the "
 					"file you have just saved."));
 	}
 }
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 /* save_as helper */
 static void
 save_as_response_cb (GtkNativeDialog *dialog,
@@ -1115,17 +1123,16 @@ save_as (GtkWidget *widget,
 
 	gtk_native_dialog_show (GTK_NATIVE_DIALOG(file_sel));
 }
-
+G_GNUC_END_IGNORE_DEPRECATIONS
 
 static void
-revert_response_cb (GtkDialog *dialog,
-		int response_id,
-		gpointer user_data)
+revert_response_cb (AdwMessageDialog *dialog,
+		const char *response,
+		GHexApplicationWindow *self)
 {
-	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
 	HexDocument *doc;
 
-	if (response_id != GTK_RESPONSE_ACCEPT)
+	if (g_strcmp0 (response, "accept") != 0)
 		goto end;
 
 	doc = hex_widget_get_document (ACTIVE_GH);
@@ -1159,27 +1166,24 @@ revert (GtkWidget *widget,
 
 	basename = g_file_get_basename (hex_document_get_file (doc));
 
-	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(self),
-			GTK_DIALOG_MODAL,
-			GTK_MESSAGE_QUESTION,
-			GTK_BUTTONS_NONE,
+	dialog = adw_message_dialog_new (GTK_WINDOW(self), NULL, NULL);
+	adw_message_dialog_format_body_markup (ADW_MESSAGE_DIALOG(dialog),
 			/* Translators: %s here is the filename the user is being asked to
 			 * confirm whether they want to revert. */
 			_("<big><b>Are you sure you want to revert %s?</b></big>\n\n"
 			"Your changes will be lost.\n\n"
 			"This action cannot be undone."),
 			basename);
-
-	gtk_dialog_add_buttons (GTK_DIALOG(dialog),
-			_("_Revert"), GTK_RESPONSE_ACCEPT,
-			_("_Go Back"), GTK_RESPONSE_REJECT,
+	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG(dialog),
+			"accept", _("_Revert"),
+			"reject", _("_Go Back"),
 			NULL);
-	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_REJECT);
+	adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG(dialog), "reject");
 
-	g_signal_connect (dialog, "response",
-			G_CALLBACK (revert_response_cb), self);
+	g_signal_connect (dialog, "response", G_CALLBACK(revert_response_cb), self);
+
+	gtk_widget_set_visible (dialog, TRUE);
 
-	gtk_widget_show (dialog);
 	g_free (basename);
 }
 			
@@ -1217,7 +1221,10 @@ new_file (GtkWidget *widget,
 	HexWidget *gh;
 
 	doc = hex_document_new ();
-	gh = HEX_WIDGET(hex_widget_new (doc));
+	gh = g_object_new (HEX_TYPE_WIDGET,
+			"document", doc,
+			"fade-zeroes", TRUE,
+			NULL);
 
 	ghex_application_window_add_hex (self, gh);
 	refresh_dialogs (self);
@@ -1226,6 +1233,7 @@ new_file (GtkWidget *widget,
 	file_loaded (doc, self);
 }
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 static void
 open_response_cb (GtkNativeDialog *dialog,
 		int resp,
@@ -1266,6 +1274,7 @@ open_file_action (GtkWidget *widget,
 
 	gtk_native_dialog_show (GTK_NATIVE_DIALOG(file_sel));
 }
+G_GNUC_END_IGNORE_DEPRECATIONS
 
 static void
 toggle_conversions (GtkWidget *widget,
@@ -1293,6 +1302,7 @@ toggle_conversions (GtkWidget *widget,
 	}
 }
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 static void
 open_help_ready_cb (GObject *source_object,
 		GAsyncResult *res,
@@ -1367,7 +1377,7 @@ error:
 					"The specific error message is:\n\n"
 					"%s"),
 				local_error->message);
-		display_error_dialog (GTK_WINDOW(self), full_errmsg);
+		display_dialog (GTK_WINDOW(self), full_errmsg);
 out:
 		g_clear_object (&file);
 		g_free (uri);
@@ -1375,7 +1385,7 @@ out:
 		g_clear_pointer (&local_error, g_error_free);
 #undef URI_PREFIX
 #else
-		display_error_dialog (GTK_WINDOW(self), local_error->message);
+		display_dialog (GTK_WINDOW(self), local_error->message);
 #endif
 	}
 }
@@ -1394,6 +1404,7 @@ open_help (GtkWidget *widget,
 			open_help_ready_cb,
 			NULL);
 }
+G_GNUC_END_IGNORE_DEPRECATIONS
 
 static void
 open_about (GtkWidget *widget,
@@ -1416,7 +1427,124 @@ open_preferences (GtkWidget *widget,
 			! gtk_widget_get_visible (self->prefs_dialog)) {
 		self->prefs_dialog = create_preferences_dialog (GTK_WINDOW(self));
 	}
-	gtk_widget_show (self->prefs_dialog);
+	gtk_widget_set_visible (self->prefs_dialog, TRUE);
+}
+
+/* Mark actions */
+
+static void
+goto_mark (GHexApplicationWindow *self, const char *key)
+{
+	HexWidgetMark *mark;
+
+	mark = g_object_get_data (G_OBJECT(ACTIVE_GH), key);
+
+	if (! mark) {
+		g_debug ("%s: No mark found at %s", __func__, key);
+		return;
+	}
+
+	hex_widget_goto_mark (ACTIVE_GH, mark);
+	gtk_widget_grab_focus (GTK_WIDGET(ACTIVE_GH));
+}
+
+static void
+set_mark (GHexApplicationWindow *self, const char *key)
+{
+	HexWidgetMark *mark;
+	gint64 ss, se;
+
+	mark = g_object_get_data (G_OBJECT(ACTIVE_GH), key);
+
+	if (mark) {
+		g_debug ("%s: Pre-existing mark found at %s - deleting", __func__, key);
+		hex_widget_delete_mark (ACTIVE_GH, mark);
+	}
+
+	hex_widget_get_selection (ACTIVE_GH, &ss, &se);
+	mark = hex_widget_add_mark (ACTIVE_GH, ss, se, /*GdkRGBA *color*)*/ NULL);
+
+	g_object_set_data (G_OBJECT(ACTIVE_GH), key, mark);
+
+	hex_widget_clear_selection (ACTIVE_GH);
+	update_gui_data (self);
+	gtk_widget_grab_focus (GTK_WIDGET(ACTIVE_GH));
+}
+
+static void
+delete_mark (GHexApplicationWindow *self, const char *key)
+{
+	HexWidgetMark *mark;
+
+	mark = g_object_get_data (G_OBJECT(ACTIVE_GH), key);
+
+	if (mark) {
+		hex_widget_delete_mark (ACTIVE_GH, mark);
+	}
+	else {
+		g_debug ("%s: No mark found at %s - ignoring", __func__, key);
+		return;
+	}
+
+	g_object_set_data (G_OBJECT(ACTIVE_GH), key, NULL);
+
+	update_gui_data (self);
+	gtk_widget_grab_focus (GTK_WIDGET(ACTIVE_GH));
+}
+
+static void
+mark_action (GtkWidget *widget, const char *action_name, GVariant *parameter)
+{
+	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(widget);
+	HexWidgetMark *mark;
+	char *mark_action;
+	int mark_num;
+	char *mark_name;
+	char *key;
+
+	if (! ACTIVE_GH)
+		return;
+
+	g_variant_get (parameter, "(sims)", &mark_action, &mark_num, &mark_name);
+	key = g_strdup_printf ("mark%d", mark_num);
+
+	if (g_strcmp0 (mark_action, "jump") == 0)
+	{
+		goto_mark (self, key);
+	}
+	else if (g_strcmp0 (mark_action, "set") == 0)
+	{
+		set_mark (self, key);
+	}
+	else if (g_strcmp0 (mark_action, "delete") == 0)
+	{
+		delete_mark (self, key);
+	}
+	else
+	{
+		g_critical ("%s: Invalid action: %s", __func__, mark_action);
+	}
+
+	g_free (mark_action);
+	g_free (mark_name);
+	g_free (key);
+}
+
+static void
+activate_mark_action (GtkWidget *widget,
+		const char *action_name,
+		GVariant *parameter)
+{
+	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(widget);
+	int mark_num = g_variant_get_int32 (parameter);
+
+	if (mark_num < 0 || mark_num > 9) {
+		g_warning ("%s: Programmer error: invalid mark number", __func__);
+		return;
+	}
+
+	if (get_pane_visible (self, PANE_DIALOG(self->mark_dialog)))
+		mark_dialog_activate_mark_num (MARK_DIALOG(self->mark_dialog), mark_num);
 }
 
 /* --- */
@@ -1456,7 +1584,7 @@ out:
 static gboolean
 get_dialog_visible (GtkWidget *widget)
 {
-	return (GTK_IS_WIDGET(widget) && gtk_widget_get_visible (widget));
+	return (GTK_IS_WIDGET (widget) && gtk_widget_get_visible (widget));
 }
 
 static gboolean
@@ -1504,6 +1632,11 @@ ghex_application_window_set_property (GObject *object,
 					g_value_get_boolean (value));
 			break;
 
+		case PROP_MARK_OPEN:
+			ghex_application_window_set_show_mark (self,
+					g_value_get_boolean (value));
+			break;
+
 		case PROP_CAN_SAVE:
 			ghex_application_window_set_can_save (self,
 					g_value_get_boolean (value));
@@ -1554,6 +1687,11 @@ ghex_application_window_get_property (GObject *object,
 					get_pane_visible (self, PANE_DIALOG(self->jump_dialog)));
 			break;
 
+		case PROP_MARK_OPEN:
+			g_value_set_boolean (value,
+					get_pane_visible (self, PANE_DIALOG(self->mark_dialog)));
+			break;
+
 		case PROP_CAN_SAVE:
 			g_value_set_boolean (value, self->can_save);
 			break;
@@ -1618,7 +1756,14 @@ ghex_application_window_init (GHexApplicationWindow *self)
 	action = g_settings_create_action (settings, GHEX_PREF_GROUP);
 	g_action_map_add_action (G_ACTION_MAP(self), action);
 
-	/* Setup tab view signals */
+	/* Setup tab view */
+
+	/* Don't trample over Ctrl+Home/End & friends. */
+	adw_tab_view_remove_shortcuts (ADW_TAB_VIEW(self->hex_tab_view),
+			ADW_TAB_VIEW_SHORTCUT_CONTROL_END |
+			ADW_TAB_VIEW_SHORTCUT_CONTROL_HOME |
+			ADW_TAB_VIEW_SHORTCUT_CONTROL_SHIFT_END |
+			ADW_TAB_VIEW_SHORTCUT_CONTROL_SHIFT_HOME);
 
 	g_signal_connect (self->hex_tab_view, "notify::selected-page",
 			G_CALLBACK(tab_view_page_changed_cb), self);
@@ -1640,24 +1785,31 @@ ghex_application_window_init (GHexApplicationWindow *self)
 	self->find_dialog = find_dialog_new ();
 	gtk_widget_set_hexpand (self->find_dialog, TRUE);
 	gtk_box_append (GTK_BOX(self->findreplace_box), self->find_dialog);
-	gtk_widget_hide (self->find_dialog);
+	gtk_widget_set_visible (self->find_dialog, FALSE);
 	g_signal_connect (self->find_dialog, "closed",
 			G_CALLBACK(pane_close_cb), self);
 
 	self->replace_dialog = replace_dialog_new ();
 	gtk_widget_set_hexpand (self->replace_dialog, TRUE);
 	gtk_box_append (GTK_BOX(self->findreplace_box), self->replace_dialog);
-	gtk_widget_hide (self->replace_dialog);
+	gtk_widget_set_visible (self->replace_dialog, FALSE);
 	g_signal_connect (self->replace_dialog, "closed",
 			G_CALLBACK(pane_close_cb), self);
 
 	self->jump_dialog = jump_dialog_new ();
 	gtk_widget_set_hexpand (self->jump_dialog, TRUE);
 	gtk_box_append (GTK_BOX(self->findreplace_box), self->jump_dialog);
-	gtk_widget_hide (self->jump_dialog);
+	gtk_widget_set_visible (self->jump_dialog, FALSE);
 	g_signal_connect (self->jump_dialog, "closed",
 			G_CALLBACK(pane_close_cb), self);
 
+	self->mark_dialog = mark_dialog_new ();
+	gtk_widget_set_hexpand (self->mark_dialog, TRUE);
+	gtk_box_append (GTK_BOX(self->findreplace_box), self->mark_dialog);
+	gtk_widget_set_visible (self->mark_dialog, FALSE);
+	g_signal_connect (self->mark_dialog, "closed",
+			G_CALLBACK(pane_close_cb), self);
+
 	hex_statusbar_clear (HEX_STATUSBAR(self->statusbar));
 
 	/* Grey out main actions at the beginning */
@@ -1670,6 +1822,7 @@ ghex_application_window_init (GHexApplicationWindow *self)
 	/* DnD */
 
 	target = gtk_drop_target_new (G_TYPE_FILE, GDK_ACTION_COPY);
+	gtk_drop_target_set_gtypes (target, (GType[1]) { GDK_TYPE_FILE_LIST }, 1);
 	g_signal_connect (target, "drop", G_CALLBACK(dnd_drop_cb), self);
 	gtk_widget_add_controller (widget, GTK_EVENT_CONTROLLER(target));
 
@@ -1707,7 +1860,6 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 
 	/* PROPERTIES */
 
-
 	properties[PROP_CHARTABLE_OPEN] =
 		g_param_spec_boolean ("chartable-open",
 			"Character table open",
@@ -1743,6 +1895,11 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 			FALSE,	/* gboolean default_value */
 			prop_flags);
 
+	properties[PROP_MARK_OPEN] =
+		g_param_spec_boolean ("mark-open", NULL, NULL,
+			FALSE,	/* gboolean default_value */
+			prop_flags);
+
 	properties[PROP_CAN_SAVE] =
 		g_param_spec_boolean ("can-save",
 			"Can save",
@@ -1759,7 +1916,6 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 
 	g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 
-
 	/* ACTIONS */
 
 	gtk_widget_class_install_action (widget_class, "ghex.new",
@@ -1814,12 +1970,23 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 			NULL,	/* GVariant string param_type */
 			open_about);
 
+	gtk_widget_class_install_action (widget_class, "ghex.mark",
+			"(sims)",
+			mark_action);
+
+	gtk_widget_class_install_action (widget_class, "ghex.activate-mark",
+			"i",
+			activate_mark_action);
+
 	gtk_widget_class_install_property_action (widget_class,
 			"ghex.find", "find-open");
 
 	gtk_widget_class_install_property_action (widget_class,
 			"ghex.replace", "replace-open");
 
+	gtk_widget_class_install_property_action (widget_class,
+			"ghex.mark-dialog", "mark-open");
+
 	gtk_widget_class_install_property_action (widget_class,
 			"ghex.jump", "jump-open");
 
@@ -1883,6 +2050,13 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 			"ghex.jump",
 			NULL);	/* no args. */
 
+	/* Ctrl+M - marks */
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_m,
+			GDK_CONTROL_MASK,
+			"ghex.mark-dialog",
+			NULL);
+
 	/* Ctrl+Shift+V - paste special */
 	gtk_widget_class_add_binding_action (widget_class,
 			GDK_KEY_v,
@@ -1939,6 +2113,130 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 			close_tab_shortcut_cb,
 			NULL);
 
+	/* Ctrl+{0-9} - activate mark spinbtn value */
+	/* r! ./generate-mark-bindings.sh
+	 */
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_0,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			0);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_0,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			0);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_1,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			1);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_1,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			1);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_2,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			2);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_2,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			2);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_3,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			3);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_3,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			3);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_4,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			4);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_4,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			4);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_5,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			5);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_5,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			5);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_6,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			6);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_6,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			6);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_7,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			7);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_7,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			7);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_8,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			8);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_8,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			8);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_9,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			9);
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_KP_9,
+			GDK_CONTROL_MASK,
+			"ghex.activate-mark",
+			"i",
+			9);
+
 	/* In case you're looking for Ctrl+PageUp & PageDown - those are baked in
 	 * via AdwTabView */
 
@@ -2050,6 +2348,10 @@ ghex_application_window_add_hex (GHexApplicationWindow *self,
 	show_hex_tab_view (self);
 }
 
+/* This nag screen is kind of stop-gap-ish anyway and will likely be removed
+ * in ghex5, so we're going to be lazy and not port it to AdwMessageDialog.
+ */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 /* Helper */
 static void
 nag_screen_response_cb (GtkDialog *nag_screen,
@@ -2096,8 +2398,9 @@ do_nag_screen (GHexApplicationWindow *self)
 				msg);
 		g_signal_connect (nag_screen, "response",
 				G_CALLBACK(nag_screen_response_cb), self);
-		gtk_widget_show (nag_screen);
+		gtk_widget_set_visible (nag_screen, TRUE);
 }
+G_GNUC_END_IGNORE_DEPRECATIONS
 
 static void
 doc_read_ready_cb (GObject *source_object,
@@ -2127,12 +2430,12 @@ doc_read_ready_cb (GObject *source_object,
 
 		if (local_error)
 		{
-			display_error_dialog (GTK_WINDOW(self), local_error->message);
+			display_dialog (GTK_WINDOW(self), local_error->message);
 			g_error_free (local_error);
 		}
 		else
 		{
-			display_error_dialog (GTK_WINDOW(self),
+			display_dialog (GTK_WINDOW(self),
 					_("There was an error reading the file."));
 		}
 	}
@@ -2185,17 +2488,20 @@ ghex_application_window_open_file (GHexApplicationWindow *self, GFile *file)
 						__func__);
 		}
 
-		gh = HEX_WIDGET(hex_widget_new (doc));
+		gh = g_object_new (HEX_TYPE_WIDGET,
+				"document", doc,
+				"fade-zeroes", TRUE,
+				NULL);
 	}
 
 	/* Display a fairly generic error message if we can't even get this far. */
 	if (!doc || !gh)
 	{
-		char *error_msg = ("There was an error loading the requested file. "
+		char *error_msg = _("There was an error loading the requested file. "
 				"The file either no longer exists, is inaccessible, "
 				"or you may not have permission to access the file.");
 
-		display_error_dialog (GTK_WINDOW(self), error_msg);
+		display_dialog (GTK_WINDOW(self), error_msg);
 
 		/* This fcn is also used to handle open operations on the cmdline. */
 		g_printerr ("%s\n", error_msg);
