@@ -52,7 +52,7 @@ static void undo_stack_ascend           (HexDocument *doc);
 static void undo_stack_free             (HexDocument *doc);
 
 #define DEFAULT_UNDO_DEPTH 1024
-#define REGEX_SEARCH_LEN 1024	/* FIXME/TODO: This is kind of lazy. Stopgap? */
+#define REGEX_SEARCH_LEN 1024
 
 /* SIGNALS */
 
@@ -123,11 +123,25 @@ G_DEFINE_BOXED_TYPE (HexDocumentFindData, hex_document_find_data,
 
 /* HexChangeData GType Definitions */
 
-/* FIXME - unused and could be unreliable */
 static HexChangeData *
 hex_change_data_copy (HexChangeData *data)
 {
-	return data;
+	HexChangeData *new = NULL;
+
+	g_return_val_if_fail (data != NULL, NULL);
+
+	new = g_new0 (HexChangeData, 1);
+
+	new->start = data->start;
+	new->end = data->end;
+	new->rep_len = data->rep_len;
+	new->lower_nibble = data->lower_nibble;
+	new->insert = data->insert;
+	new->type = data->type;
+	new->v_string = g_strdup (data->v_string);
+	new->v_byte = data->v_byte;
+
+	return new;
 }
 
 G_DEFINE_BOXED_TYPE (HexChangeData, hex_change_data,
@@ -198,10 +212,11 @@ hex_document_get_property (GObject *object,
 	switch (property_id)
 	{
 		case GFILE:
-			/* --- */
+			g_value_set_object (value, hex_document_get_file (doc));
 			break;
 
 		case BUFFER:
+			g_value_set_object (value, hex_document_get_buffer (doc));
 			break;
 
 		default:
@@ -455,11 +470,13 @@ static void
 hex_document_init (HexDocument *doc)
 {
 	HexBuffer *try_buf = NULL;
+	const char *default_buf;
 
-	/* FIXME - make this a setting; preferred default backend, or
-	 * what-have-you.
-	 */
-	try_buf = hex_buffer_util_new ("mmap", NULL);
+	default_buf = g_getenv ("HEX_BUFFER");
+	if (! default_buf)
+		default_buf = "mmap";
+
+	try_buf = hex_buffer_util_new (default_buf, NULL);
 	if (! try_buf)
 		try_buf = hex_buffer_util_new (NULL, NULL);
 
@@ -477,6 +494,11 @@ hex_document_init (HexDocument *doc)
  *
  * Create a new empty [class@Hex.Document] object.
  *
+ * Since 4.6, the HEX_BUFFER environment variable can optionally be set to
+ * specify which backend should be tried first as the default. Otherwise, the
+ * `mmap` buffer will be attempted to be loaded if available, and if it is not,
+ * it will fall back to the `malloc` buffer backend.
+ *
  * Returns: a new [class@Hex.Document] object.
  */
 HexDocument *
@@ -497,6 +519,19 @@ hex_document_new (void)
 gboolean
 hex_document_set_file (HexDocument *doc, GFile *file)
 {
+	g_return_val_if_fail (HEX_IS_DOCUMENT (doc), FALSE);
+	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+	/* Only the malloc backend can open 0-length files */
+	if (hex_buffer_util_get_file_size (file) == 0)
+	{
+		HexBuffer *buf;
+
+		g_debug ("%s: Zero-length file detected. Attempting to set `malloc` buffer.", __func__);
+		buf = hex_buffer_util_new ("malloc", file);
+		hex_document_set_buffer (doc, buf);
+	}
+
 	if (! hex_buffer_set_file (doc->buffer, file)) {
 		g_debug ("%s: Invalid file", __func__);
 		return FALSE;
@@ -643,7 +678,7 @@ hex_document_set_byte (HexDocument *doc, char val, gint64 offset,
  * @offset: offset in bytes within the payload
  * @len: length in bytes of the data to be set
  * @rep_len: amount of bytes to replace/overwrite (if any)
- * @data: (array length=len) (transfer full): a pointer to the data being
+ * @data: (array length=len) (element-type gint8): a pointer to the data being
  *   provided
  * @undoable: whether the operation should be undoable
  *
@@ -781,7 +816,6 @@ hex_document_read_async (HexDocument *doc,
 		GAsyncReadyCallback callback,
 		gpointer user_data)
 {
-	static HexChangeData change_data;
 	GTask *task;
 	gint64 payload;
 
@@ -1334,8 +1368,8 @@ out:
 /**
  * hex_document_compare_data:
  * @doc: a [class@Hex.Document] object
- * @what: (array length=len): a pointer to the data to compare to data within
- *   the #HexDocument
+ * @what: (array length=len) (element-type gint8): a pointer to the data to
+ *   compare to data within the #HexDocument
  * @pos: offset position of the #HexDocument data to compare with @what
  * @len: size of the #HexDocument data to compare with @what, in bytes
  *
@@ -1404,8 +1438,8 @@ hex_document_find_forward_full (HexDocument *doc,
  * hex_document_find_forward:
  * @doc: a [class@Hex.Document] object
  * @start: starting offset byte of the payload to commence the search
- * @what: (array length=len): a pointer to the data to search within the
- *   #HexDocument
+ * @what: (array length=len) (element-type gint8): a pointer to the data to
+ *   search within the #HexDocument
  * @len: length in bytes of the data to be searched for
  * @offset: (out): offset of the found string, if the method returns %TRUE
  * 
@@ -1528,8 +1562,8 @@ FIND_FULL_ASYNC_TEMPLATE(hex_document_find_forward_full_async,
  * hex_document_find_forward_async:
  * @doc: a [class@Hex.Document] object
  * @start: starting offset byte of the payload to commence the search
- * @what: (array length=len): a pointer to the data to search within the
- *   #HexDocument
+ * @what: (array length=len) (element-type gint8): a pointer to the data to
+ *   search within the #HexDocument
  * @len: length in bytes of the data to be searched for
  * @offset: (out): offset of the found string, if the method returns %TRUE
  * @found_msg: message intended to be displayed by the client if the string
@@ -1617,8 +1651,8 @@ hex_document_find_backward_full (HexDocument *doc,
  * hex_document_find_backward:
  * @doc: a [class@Hex.Document] object
  * @start: starting offset byte of the payload to commence the search
- * @what: (array length=len): a pointer to the data to search within the
- *   #HexDocument
+ * @what: (array length=len) (element-type gint8): a pointer to the data to
+ *   search within the #HexDocument
  * @len: length in bytes of the data to be searched for
  * @offset: (out): offset of the found string, if the method returns %TRUE
  * 
@@ -1689,8 +1723,8 @@ FIND_FULL_ASYNC_TEMPLATE(hex_document_find_backward_full_async,
  * hex_document_find_backward_async:
  * @doc: a [class@Hex.Document] object
  * @start: starting offset byte of the payload to commence the search
- * @what: (array length=len): a pointer to the data to search within the
- *   #HexDocument
+ * @what: (array length=len) (element-type gint8): a pointer to the data to
+ *   search within the #HexDocument
  * @len: length in bytes of the data to be searched for
  * @offset: (out): offset of the found string, if the method returns %TRUE
  * @found_msg: message intended to be displayed by the client if the string
@@ -1929,7 +1963,7 @@ hex_document_get_file (HexDocument *doc)
 /**
  * hex_document_set_buffer:
  * @doc: a [class@Hex.Document] object
- * @buf: [iface@Hex.Buffer]
+ * @buf: (transfer full): [iface@Hex.Buffer]
  *
  * Set the [iface@Hex.Buffer] connected with the #HexDocument.
  *
